JSEPackage("ViaMichelin.Api.Model.Destination");

JSEImportApi("ViaMichelin.Api.Model.AbstractObject");

/**
 * Modèle utilisé pour les Destinations
 * <br>Il est indispensable que l'id soit renseigné afin de pouvoir utiliser ce modèle dans les différents composants de l'API
 * @class ViaMichelin.Api.Model.Destination
 * @param {Object} conf Parametre json pour instancier le modèle
 * @example
 * Exemple de json pour les paramètres : 
 * boundary : {
 *    lon0 : 11,
 *    lat0 : 22,
 *    lon1 : 33,
 *    lat1 : 44
 * },
 * center : {
 *    lon : 55,
 *    lat : 66
 * },
 * cotation : 0,
 * flag : "/img/data/flags_desti/esp.png",
 * id : "E2000074fra",
 * isCity : false,
 * isCountry : true,
 * isoCountry : "ESP",
 * link : "web/Destination/Espagne",
 * name : "Espagne",
 * reflexId : "0"
 */
ViaMichelin.Api.Model.Destination = function (conf) {
	this.extend(ViaMichelin.Api.Model.AbstractObject);
	this.update(conf);
};

ViaMichelin.Api.Model.Destination.prototype = {
	
	/**
	 * Contient l'id de la destination
	 */
	id		: null,
	/**
	 * Contient le nom de la destination
	 */
	name	: "",
	
	/**
	 * Mise à jour des champs de l'objet
	 * @param {Object} desti json définissant une destination
	 */
	update : function (desti) {
		for(var item in desti){
			this[item] = desti[item];
		}
	},
	
	
	// Accesseurs
	// ----------
	
	/**
 	 * Retourne l'id du poi
	 * @return {String} Id du poi
	 */
	getId : function() {
		return this.id;
	},
	
	/**
	 * Retourne le nom du poi selon une liste de priorité des sources
	 * @return {String} Nom du poi
	 */
	getName : function(){
		return this.name;
	},
	
	/**
	 * Retourne le type de poi
	 * @return {String} Type de poi
	 */
	getType : function () {
		return "DESTINATION";
	},
	
	/**
	 * Retourne l'URL de la datasheet de la destination
	 * <br>Note : Basé sur getNameForURL, comporte donc les mêmes limites
	 * @return {String} L'url de la datasheet 
	 */
	getURL : function () {
		return null;
		/*
		if ($isDefined(ViaMichelin.Api.Constants.System.ApiVMVWebRoot)) {
			var url = ViaMichelin.Api.Constants.System.ApiVMVWebRoot;
		} else if ($isDefined(ViaMichelin.Api.Constants.System.ApiVMWebRoot)) {
			var url = ViaMichelin.Api.Constants.System.ApiVMWebRoot
		} else {
			return "";
		}
		url += "/web/Destination/" + this.getNameForURL();
		return url;
		*/
	},
	
	/**
	 * Retourne le nom de la destination tel qu'il doit apparaître dans l'URL de sa datasheet, 
	 * en supprimant tout caractère spécial
	 * <br>Ne fonctionne que pour les caractères les plus courants, ceux des 5 langues du site voyage
	 * @return {String} Le nom formatté
	 */
	getNameForURL : function () {
		var ancestry = this.ancestry;
		var special_chars = "àáâäçèéêëìíîïñòóôöùúûüýÿ".split("");
		var classic_chars = "aaaaceeeeiiiinoooouuuuyy".split("");
		var url = (ancestry ? (ancestry + "-") : "") + this.name;
		for (var i = 0; i < special_chars.length; i++) url = url.replaceAll(special_chars[i], classic_chars[i]);
		url = url.replaceAll("-", "_");
		url = url.replaceAll("/", "-");
		return url;
	},
	
	/**
	 * Retourne le type de l'objet qu'est le dossier
	 * @return {String} "ressource"
	 */
	getObjectType : function () {
		return "ressource";
	},
	
	/**
	 * Retourne le sous-type de l'objet qu'est le dossier
	 * @return {String} "DESTI"
	 */
	getObjectSubType : function () {
		return "DESTI";
	},
	
	/**
	 * Permet de vérifier que la destination est valide
	 * <br>Pour être considéré comme valide, il doit contenir les paramètres suivants :
	 * <br>- id
	 * @return {Boolean} true si la destination est valide
	 */
	isValid : function () {
		return ($isNotEmpty(this.id));
	},
	
	/**
	 * Retourne le nom de la classe
	 * @ignore
	 * @return {String} "[ViaMichelin.Api.Model.Destination]"
	 */
	toString : function(){
		return "[ViaMichelin.Api.Model.Destination]";
	}
};