JSEPackage("ViaMichelin.Api.Model.Itinerary");

JSEImportApi("ViaMichelin.Api.Map.PolyLine");
JSEImportApi("ViaMichelin.Api.Map.ComplexLayer");
JSEImportApi("ViaMichelin.Api.Model.Itinerary.Header");
JSEImportApi("ViaMichelin.Api.Model.Itinerary.ItineraryTrace");
JSEImportApi("ViaMichelin.Api.Model.Itinerary.RoadSheetV1");
JSEImportApi("ViaMichelin.Api.Model.Itinerary.RoadSheet");
JSEImportApi("ViaMichelin.Api.Constants.Itinerary");

JSEImportTrad("ViaMichelin.Api.Trad.Itinerary.RoadSheet");

/**
 * Modèle de données pour un itinéraire
 * @class ViaMichelin.Api.Model.Itinerary
 * @constructor
 */
ViaMichelin.Api.Model.Itinerary = function(conf) {
	//this.prepare();
	this.update(conf);
};

ViaMichelin.Api.Model.Itinerary.prototype = {
	_request : null,
	/**
	 * Entête de l'itinéraire.
	 * @type ViaMichelin.Api.Model.Itinerary.Header
	 */
	header 	: null,
	/**
	 * Ititrace de l'itinéraire.
	 * @type Array
	 */
	itiTrace: null,
	itiTraceStyle : {
		strokeColor 	: "purple",
		strokeOpacity	: 0.5,
		strokeWeight	: 7,
		type			: "polyLine",
		styleOver			: {
			strokeColor 	: "purple",
			strokeOpacity	: 0.9,
			strokeWeight	: 5
		}
	},
	/**
	 * Feuille de route.
	 * @type ViaMichelin.Api.Model.Itinerary.RoadSheet
	 */
	roadSheet	: null,
	//vehiculeType : ["CAR","TRUCK","","","CARAVAN","CAR","CAR","CAR","CAR","CAR","CAR"],
	vehiculeType : ["CAR","pl2","","","MOTO","CAR","CAR","CAR","CAR","CAR","CAR"],
	/**
	 * @type ViaMichelin.Api.Map.ComplexLayer
	 */
	layer	: null,
	schema	: null,
	/**
	 * Index par défaut du choix des itinéraires alternatifs
	 * @type Number
	 */
	SUMMARY_DEFAULT : 0,
	/**
	 * Retourne l'objet de départ
	 * @return {Object}
	 */
	getArrival : function(){
		return this._request.steps[this._request.steps.length-1];
	},
	/**
	 * Retourne l'objet de départ
	 * @return {Object}
	 */
	getDeparture : function(){
		return this._request.steps[0];
	},
	/**
	 * Retourne l'unité de distance dans laquelle l'itinéraire a été calculé.
	 * @return {String}
	 */
	getDistanceUnit : function(){
		return this._request.distUnit;
	},
	/**
	 * Retourne la liste complète des étapes (départ,étapes,arrivé)
	 * @return {Array}
	 */
	getAllSteps : function(){
		return this._request.steps;
	},
	/**
	 * Retourne les émissions de CO2 sur le parcours (en kg)
	 * @param {String} type : type de carburant {@link ViaMichelin.Api.Constants.Itinerary}.FUEL_TYPE
	 * @param {Number} indice : indice de l'itinéraire
	 * @return {Number}
	 */
	getCO2Emissions : function(type, indice){
		if (!$isDefined(indice)) indice = 0;
		
		// Calcul de la consommation globale en litre
		var litres = (this.getConsumptionCost(indice) / this._request.fuelCost);
		
		switch(type) {
			case ViaMichelin.Api.Constants.Itinerary.FUEL_TYPE.PETROL :
				return litres * 2.28;
				break;
			case ViaMichelin.Api.Constants.Itinerary.FUEL_TYPE.LPG :
				return litres * 1.66;
				break;
			case ViaMichelin.Api.Constants.Itinerary.FUEL_TYPE.DIESEL :				
				return litres * 2.6;
				break;
		}
		return 0;
	},
	/**
	 * Retourne les couts de consommation de carburant
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {int}
	 */
	getCCZCost : function(indice){
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.getCCZCost(indice, this._request.wCaravan, this.vehiculeType[this._request.veht]);
		}
	},
	/**
	 * Retourne les couts de consommation de carburant
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getConsumptionCost : function(indice){
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].consumption;
		}
	},
	/**
	 * Retourn le type de monnaie
	 * @return {String}
	 */
	getCurrency : function() {
		return (this._request.currency);
	},
	getDevise : function(){
		return this._request.currency;
	},
	/**
	 * Retourne la distance parcourue sur autoroute
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getDistanceOnMotorway : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].motorwayDist;
		}
		return 0;
	},
	/**
	 * Retourne l'écotaxe
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getEcoTax : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].ecoTax;
		}
		return 0;
	},
	/**
	 * Retourne la distance utilisé pour l'écotaxe
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getEcoTaxDistance : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].ecoTaxDist;
		}
		return 0;
	},
	/**
	 * Retourne les indemnités
	 * @return {int}
	 */
	getIndemnity : function () {
		return this._request.indemnity;
	},
	/**
	 * Retourne le type de vehicule utilisé sous la forme d'un int
	 * @return {int}
	 */
	getIndiceVehiculeType : function(){
		return this._request.veht;
	},
	/**
	 * Retourne le type d'itinéraire
	 * @return {int}
	 */
	getItineraryType : function(){
		return this.header.itiType;
	},
	/**
	 * Retourne les styles utilisées pour l'ititrace
	 * @return {Object}
	 */
	getItiTraceStyle : function() {
		var conf = {};
		for (var item in this.itiTraceStyle) {
			if (typeof(this.itiTraceStyle[item]) == "object" && !$isDefined(item.length)) {
				conf[item] = {};
				for (var element in this.itiTraceStyle[item]) {
					conf[item][element] = this.itiTraceStyle[item][element];
				}
			} else {
				conf[item] = this.itiTraceStyle[item];
			}
		}
		
		return conf;
	},
	/**
	 * Retourne un itinéraire sous la forme d'un ViaMichelin.Api.Map.PolyLine
	 * @param {object} initialConf
	 * @param {String} initialConf.strokeColor
	 * @param {Number} initialConf.strokeOpacity
	 * @param {Number} initialConf.strokeWeight
	 * @param {String} initialConf.type
	 * @param {Object} initialConf.styleOver
	 * @param {String} initialConf.styleOver.strokeColor
	 * @param {Number} initialConf.styleOver.strokeOpacity
	 * @param {Number} initialConf.styleOver.strokeWeight
	 * @param {Boolean} preventDefaultEventHandlers
	 * @return {ViaMichelin.Api.Map.PolyLine}
	 */
	getItiTraceAsVMPolyLine : function(initialConf, preventDefaultEventHandlers){
		try {
			var conf = this.getItiTraceStyle();
			conf["coords"] = this.itiTrace;
			
			if($isDefined(initialConf)) {
				for (var item in initialConf) {
					if (item == "styleOver") {
						for (var element in initialConf["styleOver"]) {
							conf["styleOver"][element] = initialConf["styleOver"][element];
						}
					} else {
						conf[item] = initialConf[item];
					}
				}
			}
			var itiLine = new ViaMichelin.Api.Map.PolyLine(conf);
			
			if(!preventDefaultEventHandlers){
				itiLine.addEventListener("onMouseover", function(){
					itiLine.setStrokeWeight(conf.styleOver.strokeWeight);
					itiLine.setStrokeOpacity(conf.styleOver.strokeOpacity);
					itiLine.setStrokeColor(conf.styleOver.strokeColor);
				}, this);
				
				itiLine.addEventListener("onMouseout", function(){
					itiLine.setStrokeWeight(conf.strokeWeight);
					itiLine.setStrokeOpacity(conf.strokeOpacity);
					itiLine.setStrokeColor(conf.strokeColor);
				}, this);
			}
			itiLine.contextMenuTemplate = "/htm/obf/iti/roadsheetContextV10.2.0.htm";
			
			/* 2010-03 Désactivé pour le moment */
			//itiLine.setContextMenuControl(true);
			itiLine.setZindex("1");
			return itiLine;
		} catch(e) {
			//console.log("/!\ Error", e);
			return false;
		}
	},
	/**
	 * Retourne un itinéraire sous la forme d'un ViaMichelin.Api.Map.ComplexLayer
	 * @param {Object} initialConf
	 * @return {ViaMichelin.Api.Map.ComplexLayer}
	 */
	getItiAsVMComplexLayer : function(initialConf){
		try {
			var ComplexLayer = new ViaMichelin.Api.Map.ComplexLayer();
			if(!$isDefined(initialConf)){
				initialConf = null;
			}
			this.itiTracePolyLine = this.getItiTraceAsVMPolyLine(initialConf);
			ComplexLayer.addLayer(this.itiTracePolyLine);
			ComplexLayer.addLayer(this.roadSheet.getLayer());
			return ComplexLayer;
		}catch(e){
			return false;
		}
	},
	/**
	 * Retourne un ensemble de layer
	 * - Iti trace
	 * - Warning
	 * - Layer spécifique à la Roadsheet
	 * @param {Object} conf
	 * @return {ViaMichelin.Api.Map.ComplexLayer}
	 */
	getLayer : function (conf) {
		if (this.layer == null && ( $isDefined(this.itiTrace) || $isDefined(conf.itiTrace) ) && $isDefined(this.roadSheet)) {
			this.layer = new ViaMichelin.Api.Map.ComplexLayer();
			this.layer.setZindex("");
			
			if ($isDefined(conf) && $isDefined(conf.itiTrace)) {
				this.layer.addLayer(this.getItiTraceAsVMPolyLine(conf.itiTrace));
			} else 
				this.layer.addLayer(this.getItiTraceAsVMPolyLine());
				
			this.layer.addLayer(this.roadSheet.getWarningsLayer());
			try{
				this.layer.addLayer(this.roadSheet.getLayer());
			}catch(error){
				//console.log("error: ", error);
			}
		}
		return this.layer;
	},
	/**
	 * Retourne le montant des taxes
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Float}
	 */
	getRoadTaxCost : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.getRoadTaxCost(indice, this.itineraryCaravan, this.vehiculeType[this._request.veht]);
		}
	},
	/**
	 * Retourne la liste des étapes
	 * @return {Array}
	 */
	getSteps : function(){
		if(this._request.steps.length >= 3){
			return this._request.steps.slice(1, this._request.steps.length-1);
		} else 
			return new Array();
	},
	/**
	 * Retourne le type de vehicule utilisé pour les couts
	 * @return {String}
	 */
	getVehiculeTypeForCost : function(){
		return this.vehiculeType[this._request.veht];
	},
	/**
	 * Retourne le type de vehicule utilisé
	 * @deprecated à supprimer apres la V11.4
	 * @return {String}
	 */
	getVehiculeType : function(){
		return this.vehiculeType[this._request.veht];
	},
	/**
	 * Retourne le temps passé sur l'autoroute.
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Float}
	 */
	getTimeOnMotorway : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].motorwayTime;
		}
	},
	/**
	 * Retourne le cout de l'itinéraire
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Float}
	 */
	getTollCost : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.getTollCost(indice, this._request.wCaravan, this.vehiculeType[this._request.veht]);
		}
	},
	/**
	 * Retourne la distance globale de l'itinéraire
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getTotalDistance : function(indice){
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].totalDist;
		}
	},
	/**
	 * Retourne le temps global de l'itinéraire
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getTotalTime : function(indice){
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].totalTime;
		}
	},
	/**
	 * Retourne la consommation de gazole pour l'itinéraire.
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getTotalGasConsumption : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].consumption;
		}
	},
	/**
	 * Retourne le temps global de l'itinéraire incluant le trafic
	 * @param {Number} [indice=ViaMichelin.Api.Model.Itinerary.SUMMARY_DEFAULT] : Spécifie l'itinéraire utilisé
	 * @return {Number}
	 */
	getTrafficTime : function(indice) {
		if(this.header != null){
			indice = ($isDefined(indice) && indice!="") ? parseInt(indice) : this.SUMMARY_DEFAULT;
			return this.header.summaries[indice].trafficTime;
		}else{
			return 0;
		}
	},
	/**
	 * Retourne le type de véhicule associé à l'itinéraire
	 * @return {int}
	 */
	getVehicleType : function(){
		return parseInt(this._request.veht);
		//return this.header.vehicle;
	},
	/**
	 * Indique si l'itineraire évite les frontières
	 * @return {boolean}
	 */
	isAvoidBorders : function(){
		return this._request.avoidBorders;
	},
	/**
	 * Indique s'il y a une caravane
	 * @return {boolean}
	 */
	isCaravan : function(){
		if (this._request.wCaravan == 1)
			return true;
		else 
			return false;
	},
	/**
	 * Indique si l'itinéraire favorise les autoroutes
	 * @return {boolean}
	 */
	isFavMotorways : function(){
		return this._request.favMotorways;
	},
	/**
	 * Clone l'objet
	 * TODO : a re-voir son principe
	 * @return ViaMichelin.Api.Model.Itinerary
	 */
	cloneComponent : function() {
		var cloneComponent = new ViaMichelin.Api.Model.Itinerary();
		
		for (var item in this) {
			if (item != "JSEwireHub" && item != "roadSheet" && item != "header" && item != "_request") {
				//console.log("- - "+ item);
				cloneComponent[item] = this[item];
			}
		}
		
		if (this._request != null) {
			cloneComponent["_request"] = {};
			for (var item in this._request) {
				cloneComponent["_request"][item] = this._request[item]; 
			}
		}
		
		// clonage du header
		if (this.header != null) {
			cloneComponent["header"] = {};
			for (var item in this.header) {
				if (item == "summaries") {
					cloneComponent["header"]["summaries"] = new Array();
					
					for (var i=0; i<this.header.summaries.length; i++) {
						var summary = this.header.summaries[i];
						cloneComponent["header"]["summaries"].push(summary);
					}
					
				} else {
					cloneComponent["header"][item] = this.header[item];
				}
			}
			
		}
		
		// clonage de roadsheet
		if (this.roadSheet != null) {
			if(this._request.version==1){
				cloneComponent.roadSheet = new ViaMichelin.Api.Model.Itinerary.RoadSheetV1(
					this.roadSheet.labels, null, this.roadSheet.unit, this.roadSheet.steps
				);
			} else {
				cloneComponent.roadSheet = this.roadSheet.clone();
			}
			
			//cloneComponent.roadSheet.itinerary = cloneComponent;
			/*
			cloneComponent.roadSheet.unplugWire(new JSEWire("onClickOnFromIti", this._onClickOnFromIti, this));
			cloneComponent.roadSheet.plugWire(new JSEWire("onClickOnFromIti", this._onClickOnFromIti, this));
			cloneComponent.roadSheet.unplugWire(new JSEWire("onClickOnToIti", this._onClickOnToIti, this));
			cloneComponent.roadSheet.plugWire(new JSEWire("onClickOnToIti", this._onClickOnToIti, this));
			cloneComponent.roadSheet.unplugWire(new JSEWire("onClickOnViaIti", this._onClickOnViaIti, this));
			cloneComponent.roadSheet.plugWire(new JSEWire("onClickOnViaIti", this._onClickOnViaIti, this));
			*/
			cloneComponent.roadSheet.partnerSelected = this.partnerSelected;
			cloneComponent.roadSheet.lang = this.roadSheet.lang;
			cloneComponent.roadSheet.zone = this.roadSheet.zone;
			if (cloneComponent.itineraryCaravan == 1) cloneComponent.roadSheet.tollVehicule = "caravan";
			if (cloneComponent.itineraryVehiculeType == 1) cloneComponent.roadSheet.tollVehicule = "moto";
			//cloneComponent.roadSheet.steps = cloneComponent.roadSheet._transform(this.fdr);			
		}
		return cloneComponent;
	},
	release : function(){
		if (this.layer != null) {
			if (this.layer.mapObject != null) {
				this.layer.mapObject.removeLayer(this.layer);
			}
			this.layer.release();
			this.layer = null;
		}
		this.header = null;
		this.itiTrace = null;
		
		if(this.roadSheet != null){
			this.roadSheet.release();
			this.roadSheet = null;
		}
	},
	toString : function(){
		var txt = "[ViaMichelin.Api.Model.Itinerary] ";
		var current = this;
		for(var ele in current){
			if(typeof(current[ele]) != "function"){
				txt+= "{"+ ele +" : "+ current[ele] +"} ,";
			}
		}
		return txt;
	},
	/**
	 * @param {Object} obj : json représentant l'itinéraire retourné principalement par l'APIREST
	 */
	update : function(obj){
		if (obj) {
			if (obj["request"]) {
				this._request = obj["request"];
			}
			
			if (obj["header"]) {
				this.header = new ViaMichelin.Api.Model.Itinerary.Header(obj["header"]);
			}
			
			if (obj["itineraryTrace"]) {
				// Petite adaptation de l'itiTrace pour l'instant
				var itiTrace = obj["itineraryTrace"].substring(2, obj["itineraryTrace"].length);
				this.itiTrace = new ViaMichelin.Api.Model.Itinerary.ItineraryTrace(itiTrace).getCoordsFromStringDirect();
			}
			
			if (obj["schema"]) {
				this.schema = obj["schema"];
			}
			
			if (obj["roadSheet"]) {
				// Choix de l'unité d'affichage km ou mi
				var unit = this._request.distUnit;
				if(unit == ViaMichelin.Api.Constants.Itinerary.DIST_UNIT.METERS){
					unit = "km";
				}
				
				if (this._request.version == 1) {
					this["roadSheet"] = new ViaMichelin.Api.Model.Itinerary.RoadSheetV1(ViaMichelin.Api.Trad.Itinerary.RoadSheet, null, unit, obj["roadSheet"][0]);			
				} else {
					var list = [];
					
					if (this.header) {
						// Traitement sur le message du header
						var messages = this.header.summaries[0].summaryMessages;
						if($isDefined(messages) && $isDefined(messages.msg)){
							var message = messages.msg.label;
							list.push(["M","","","",""+ messages.msg.label +"<br/>","",""]);
						}
					}
					
					for(var i=0;i<obj["roadSheet"][0].length;i++){
						list.push(obj["roadSheet"][0][i]);
					}
					
					var custom = this._request.pois;
					if($isDefined(custom)) custom = this._request.pois.customizedPictos;
					else custom = null;
					
					var customizedMap = {
						step : this._request.stepMapOpt,
						full : this._request.fullMapOpt
					};
					
					// Calcul du type de couts
					var tollVehicule = null;
					switch(parseInt(this._request.veht)){
						case ViaMichelin.Api.Constants.Itinerary.VEHICULE_TYPE.CAR:
							if(this._request.wCaravan)
								tollVehicule = ViaMichelin.Api.Constants.Itinerary.TOLL_CAT.CARAVAN;
							else
								tollVehicule = ViaMichelin.Api.Constants.Itinerary.TOLL_CAT.CAR;
							break;
						case ViaMichelin.Api.Constants.Itinerary.VEHICULE_TYPE.TRUCK:
							if ($isNotEmpty(this._request.truckOpts) && $isNotEmpty(this._request.truckOpts.axles)) {
								var value = parseInt(this._request.truckOpts.axles);
								if(value>5) value=2;
								tollVehicule = ViaMichelin.Api.Constants.Itinerary.TOLL_CAT["PL"+ value];
							} else {
								tollVehicule = ViaMichelin.Api.Constants.Itinerary.TOLL_CAT["PL2"];
							}
							
							break;
						case ViaMichelin.Api.Constants.Itinerary.VEHICULE_TYPE.ON_FOOT:
						case ViaMichelin.Api.Constants.Itinerary.VEHICULE_TYPE.CYCLE:
							
							break;
						case ViaMichelin.Api.Constants.Itinerary.VEHICULE_TYPE.MOTO:
							tollVehicule = ViaMichelin.Api.Constants.Itinerary.TOLL_CAT.MOTO;
							break;
					}
					
					this["roadSheet"] = new ViaMichelin.Api.Model.Itinerary.RoadSheet(ViaMichelin.Api.Trad.Itinerary.RoadSheet, null, unit, list, this["schema"], custom, customizedMap, tollVehicule);
				}
			}
		}
	}
};