JSEPackage("ViaMichelin.Api.Model.MPMPoi");

JSEImportApi("ViaMichelin.Api.Constants.MPMPoi");
JSEImportApi("ViaMichelin.Api.Model.AbstractObject");
JSEImportApi("ViaMichelin.Api.Util");



/**
 * Définition d'un modèle de poi MPM
 * @class    ViaMichelin.Api.Model.MPMPoi
 * @extends  ViaMichelin.Api.Model.AbstractObject
 * @requires ViaMichelin.Api.Constants.MPMPoi
 * @example 
 * <i>//Pour instancier un MPMPoi, il faut transmettre au constructeur un poi au format json</i>
 * var mpmpoi = new ViaMichelin.Api.Model.MPMPoi(poi_json);
 * @example 
 * <i>//Autre méthode</i>
 * var mpmpoi = new ViaMichelin.Api.Model.MPMPoi();
 * mpmpoi.update(poi_json);
 * @param    {Object} conf Données du poi à ajouter à la classe
 */
ViaMichelin.Api.Model.MPMPoi = function (conf) {
	this.nature = "poi";
	this.extend(ViaMichelin.Api.Model.AbstractObject);
	this.update(conf);
};

ViaMichelin.Api.Model.MPMPoi.prototype = {
	
	_vouchersList : null,
	
	/**
	 * Permet de définir les différentes propriétés du poi
	 * @param {Object} poi Poi au format json
	 */
	update : function (poi) {
		for (var item in poi) {
			this[item] = poi[item];
		}
	},
	
	
	// Accesseurs
	// ----------
	
	/**
 	 * Retourne l'id du poi
	 * @return {String} Id du poi
	 */
	getId : function () {
		return this.poi_id;
	},
	
	/**
	 * Retourne le type de poi
	 * @return {String} Type de poi
	 */
	getType : function () {
		return this.type;
	},
	
	/**
	 * Retourne le nom du poi selon une liste de priorité des sources
	 * @return {String} Nom du poi
	 */
	getName : function () {
		var priorities = ["AGG", "PRO", "GA", "LC", "UGC"];
		var name = this._getFieldWithPriorities("name", priorities);
		return (name != null) ? name : "";
	},
	
	/**
	 * Retourne l'adresse du poi selon une liste de priorité des sources
	 * @return {String} Adresse du poi
	 */
	getAddress : function () {
		var priorities = ["AGG", "PRO", "GA", "LC", "UGC"];
		var address = this._getFieldWithPriorities("address", priorities);
		return (address != null) ? address : "";
	},
	
	/**
	 * Retourne la ville du poi selon une liste de priorité des sources
	 * @return {String} Ville du poi
	 */
	getCity : function () {
		var priorities = ["AGG", "PRO", "GA", "LC", "UGC"];
		var city = this._getFieldWithPriorities("city", priorities);
		return (city != null) ? city : "";
	},
	
	/**
	 * Retourne le code postal du poi selon une liste de priorité des sources
	 * @return {String} Code postal du poi
	 */
	getPostcode : function () {
		var priorities = ["AGG", "PRO", "GA", "LC", "UGC"];
		var postcode = this._getFieldWithPriorities("postcode", priorities);
		return (postcode != null) ? postcode : "";
	},
	
	/**
	 * Retourne l'URL qui permet d'accèder au poi
	 * <br>Modèle d'URL restaurant : {URL_GM21}/restaurant/{pays}/{ville}/{nom_restaurant}/{id_adn}
	 * @return {String} URL permettant d'accèder à la page du poi
	 */
	getURL : function () {
		if (!$isDefined(ViaMichelin.Api.Constants.System.ApiGM21WebRoot))
			ViaMichelin.Api.Constants.System.ApiGM21WebRoot = "http://restaurant.michelin.fr/";
		
		var url;
		if (this.isPoiRestaurant()) {
			url = ViaMichelin.Api.Constants.System.ApiGM21WebRoot
				+ "/restaurant/p/v/n"
				+ "/" + this.getId();
			return url;
		} else {
			return "";
		}
	},
	
	/**
	 * Retourne l'image à afficher par défaut pour le POI MPM si trouvée
	 * @return {Object} Objet représentant le média
	 */
	getDefaultPhoto : function () {
		var priorities = ["AGG", "PRO", "GA", "LC", "UGC"],
			i, j;
		var medias = this._getFieldWithPriorities("medias", priorities);
		if (!medias) {
			return null;
		}
		for (i = 0; i < priorities.length; i++) {
			for (j = 0; j < medias.length; j++) {
				if (medias[j].type == "image/jpeg" && medias[j].provider == priorities[i]) {
					return medias[j];
				}
			}
		}
		return null;
	},
	
	/**
	 * Retourne le type d'objet qu'est le poi MPM
	 * @return {String} Retourne le type d'objet du poi MPM
	 */
	getObjectType : function () {
		return "poi";
	},
	
	/**
	 * Retourne le sous-type d'objet qu'est le poi MPM
	 * @return {String} Retourne le sous-type d'objet du poi MPM
	 */
	getObjectSubType : function () {
		return "_";
	},
	
	/**
	 * Retourne la liste des vouchers formatée en objets
	 */
	getVouchersList : function () {
		if (this._vouchersList === null) this._buildVouchers();
		return this._vouchersList;
	},
	
	/**
	 * Permet de savoir si le poi est dans la selection du guide Michelin
	 * @return {Boolean} true si le poi est dans la selection du guide Michelin
	 */
	isMichelinGuideSelection : function () {
		var priorities = ["AGG", "GA"];
		var mg_selection = this._getFieldWithPriorities("michelin_guide_selection", priorities);
		return (mg_selection == 1);
	},
	
	/**
	 * Permet de vérifier que le poi est valide
	 * <br>Pour être considéré comme valide, il doit contenir les paramètres suivants :
	 * <br>- poi_id
	 * <br>- type
	 * <br>- datasheets
	 * <br>- datasheets.lengh > 0
	 * @return {Boolean} true si le poi est valide
	 */
	isValid : function () {
		return ($isNotEmpty(this.poi_id) && $isNotEmpty(this.type) && $isNotEmpty(this.datasheets) && this.datasheets.length > 0);
	},
	
	// Méthodes permettant de tester le type de poi
	// --------------------------------------------
	
	/**
	 * Pemet de savoir si le poi correspond à un restaurant
	 * @return {Boolean} true si le poi est un restaurant, false sinon
	 */
	isPoiRestaurant : function() {
		return (this.type == ViaMichelin.Api.Constants.MPMPoi.RESTAURANT);
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un hotel
	 * @return {Boolean} true si le poi est un hotel, false sinon
	 */
	isPoiHotel : function() {
		return (this.type == ViaMichelin.Api.Constants.MPMPoi.HOTEL);
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un site touristique
	 * @return {Boolean} true si le poi est un site touristique, false sinon
	 */
	isPoiTourism : function() {
		return (this.type == ViaMichelin.Api.Constants.MPMPoi.TOURISM);
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un loisir
	 * @return {Boolean} true si le poi est un loisir, false sinon
	 */
	isPoiLeisure : function() {
		return (this.type == ViaMichelin.Api.Constants.MPMPoi.LEISURE);
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un loisir
	 * @deprecated Utiliser isPoiLeisure à la place
	 * @return {Boolean} true si le poi est un loisir, false sinon
	 */
	isPoiLoisir : function() {
		return (this.type == ViaMichelin.Api.Constants.MPMPoi.LEISURE);
	},
	
	
	// Méthodes privées
	// ----------------
	
	/**
	 * Structure les vouchers en un objet
	 * @private
	 */
	_buildVouchers : function () {
		this._vouchersList = [];
		var vouchersStr = this._getFieldWithPriorities("vouchers", ["AGG", "PRO", "GA", "LC", "UGC"]);;
		if (!$isNotEmpty(vouchersStr)) return;
		
		var domObject = ViaMichelin.Api.Util.getDomObjectFromXMLString(vouchersStr),
			vouchers, voucher = null, i, j, node,
			voucherObj;
		vouchers = domObject.firstChild.childNodes;
		// on recherche le voucher correspondant à l'id passé en paramètre
		for (i = 0; i < vouchers.length; i++) {
			voucher = vouchers[i];
			voucherObj = {};
			if (!voucher.getAttribute("id")) continue;
			voucherObj.id = voucher.getAttribute("id");
			
			if (voucher.getAttribute("type"))
				voucherObj.type = voucher.getAttribute("type");
			if (voucher.getAttribute("start"))
				voucherObj.start = new Date(voucher.getAttribute("start").replaceAll("-", "/") + " 00:00:00");
			if (voucher.getAttribute("end"))
				voucherObj.end = new Date(voucher.getAttribute("end").replaceAll("-", "/") + " 00:00:00");
			if (voucher.getAttribute("service"))
				voucherObj.service = voucher.getAttribute("service");
			if (voucher.getAttribute("provider"))
				voucherObj.provider = voucher.getAttribute("provider");
			
			voucherloop : for (j = 0; j < voucher.childNodes.length; j++) {
				node = voucher.childNodes[j];
				switch (node.nodeName.toLowerCase()) {
					case "name":
						voucherObj.name = node.firstChild.nodeValue;
						break;
					case "constraints":
						this._parseVoucherConstraints(node, voucherObj);
						break;
					case "inclusions":
						this._parseVoucherInclusions(node, voucherObj);
						break;
					default:
						continue voucherloop;
				}
			}
			this._vouchersList.push(voucherObj);
		}
	},
	
	/**
	 * Permet de sortir les contraintes de réservations
	 * @private
	 * @param {DOMElement} constraints Le noeud listant les contraintes
	 * @param {Object}     voucherObj  Object décrivant le voucher
	 */
	_parseVoucherConstraints : function (constraints, voucherObj) {
		var constraint, i;
		for (i = 0; i < constraints.childNodes.length; i++) {
			constraint = constraints.childNodes[i];
			if (constraint.nodeName.toLowerCase() !== "constraint") continue;
			switch (constraint.getAttribute("type")) {
				case "people":
					voucherObj.people = {};
					if (constraint.getAttribute("min")) {
						voucherObj.people.min = Number(constraint.getAttribute("min"));
					}
					if (constraint.getAttribute("max")) {
						voucherObj.people.max = Number(constraint.getAttribute("max"));
					}
					break;
				case "couple":
					voucherObj.people = {min:2, max:2};
					break;
			}
		}
	},
	
	/**
	 * Permet de sortir les éléments inclus à la réservation
	 * @private
	 * @param {DOMElement} inclusions Le noeud listant les inclusions
	 * @param {Object}     voucherObj Object décrivant le voucher
	 */
	_parseVoucherInclusions : function (inclusions, voucherObj) {
		var inclusion, subtag, i, j;
		for (i = 0; i < inclusions.childNodes.length; i++) {
			inclusion = inclusions.childNodes[i];
			if (inclusion.nodeName.toLowerCase() !== "inclusion") continue;
			for (j = 0; j < inclusion.childNodes.length; j++) {
				subtag = inclusion.childNodes[j];
				switch (subtag.nodeName.toLowerCase()) {
					case "detail":
						voucherObj.detail = $isDefined(subtag.firstChild) ? subtag.firstChild.nodeValue : "";
						break;
				}
			}
		}
	},
	
	/**
	 * Retourne la datasheet du provider spécifié
	 * @private
	 * @param  {String} provider Nom du provider
	 * @return {Object} La datasheet du provider spécifié
	 */
	_getDatasheetByProvider : function (provider) {
		if ($isDefined(this.datasheets)) {
			for (var i = 0; i < this.datasheets.length; i++) {
				if ((provider=="AGG" && this.datasheets[i].provider.indexOf() ) || this.datasheets[i].provider == provider)
					return this.datasheets[i];
			}
		}
		return null;
	},
	
	/**
	 * Retourne la propriété de la datasheet du provider spécifié
	 * @private
	 * @param  {String} provider Le provider de la datasheet concernée
	 * @param  {String} property Le nom de la propriété à retourner
	 * @return {mixed}  La propriété de la datasheet du provider spécifié
	 */
	_getDatasheetPropertyByProvider : function (provider, property) {
		var datasheet = this._getDatasheetByProvider(provider);
		if (datasheet != null && $isDefined(datasheet[property]))
			return datasheet[property];
		return null;
	},
	
	/**
	 * Retourne la valeur d'un champ selon une liste de priorités des datasheets
	 * @private
	 * @param  {String} field      Nom du champ à retourner
	 * @param  {Array}  priorities Liste des providers par priorité 
	 * @return {mixed}  La valeur du champ dans la bonne datasheet
	 */
	_getFieldWithPriorities : function (field, priorities) {
		var value, i, provider;
		if (!$isDefined(this.datasheets)) return null;
		if (this.datasheets.length == 1) {
			return this.datasheets[0][field];
		} else {
			for (i = 0; i < priorities.length; i++) {
				provider = ViaMichelin.Api.Constants.MPMPoi.PROVIDERS[priorities[i]];
				if ((value = this._getDatasheetPropertyByProvider(provider, field)) != null) 
					return value;
			}
		}
		return null;
	},
	
	/**
	 * @ignore
	 */
	toString : function() {
		return "[ViaMichelin.Api.Model.MPMPoi]";
	}
};