JSEPackage("ViaMichelin.Api.Model.MUPPoi");

JSEImportApi("ViaMichelin.Api.Model.AbstractObject");
JSEImportApi("ViaMichelin.Api.Constants.MPMPoi");

/**
 * Définition d'un modèle de poi MUP
 * @class ViaMichelin.Api.Model.MUPPoi
 * @param {Object} conf Poi au format json 
 * @extends  ViaMichelin.Api.Model.AbstractObject
 * @requires ViaMichelin.Api.Constants.MPMPoi
 * @example 
 * <i>//Pour instancier un MUPPoi, il faut transmettre au constructeur un poi au format json</i>
 * var muppoi = new ViaMichelin.Api.Model.MUPPoi(poi_json);
 * @example 
 * <i>//Autre méthode</i>
 * var muppoi = new ViaMichelin.Api.Model.MUPPoi();
 * muppoi.update(poi_json);
 */
ViaMichelin.Api.Model.MUPPoi = function (conf) {
	this.nature = "ressource";
	this.extend(ViaMichelin.Api.Model.AbstractObject);
	this.update(conf);
};

ViaMichelin.Api.Model.MUPPoi.prototype = {
	
	/**
	 * Permet de définir les différentes propriétés du poi
	 * @param {Object} poi Poi au format json
	 */
	update : function(poi){
		for(var item in poi){
			this[item] = poi[item];
		}
	},
	
	
	// Accesseurs
	// ----------
	
	/**
 	 * Retourne l'id du poi
	 * @return {String} Id du poi
	 */
	getId : function() {
		return this.id;
	},
	
	/**
	 * Retourne le type de poi
	 * @return {String} Type de poi
	 */
	getType : function () {
		if (this.isPoiRestaurant())
			return ViaMichelin.Api.Constants.MPMPoi.RESTAURANT;
		else if (this.isPoiHotel())
			return ViaMichelin.Api.Constants.MPMPoi.HOTEL;
		else if (this.isPoiTourism())
			return ViaMichelin.Api.Constants.MPMPoi.TOURISM;
		else if (this.isPoiLeisure())
			return ViaMichelin.Api.Constants.MPMPoi.LEISURE;
		else if (this.isPoiMagazine())
			return ViaMichelin.Api.Constants.MPMPoi.MAGAZINE;
		else
			return null;
	},
	
	/**
	 * Retourne le nom du poi
	 * @return {String} Nom du poi
	 */
	getName : function () {
		return $isDefined(this.name) ? this.name : "";
	},
	
	/**
	 * Retourne la ville du poi
	 * @return {String} Ville du poi
	 */
	getAddress : function () {
		return $isDefined(this.location.formattedAddressLine) ? this.location.formattedAddressLine : "";
	},
	
	/**
	 * Retourne le code postal du poi
	 * @return {String} Code postal du poi
	 */
	getCity : function () {
		return $isDefined(this.location.city) ? this.location.city : "";
	},
	
	/**
	 * Retourne le code postal du poi
	 * @return {String} Code postal du poi
	 */
	getPostcode : function () {
		return $isDefined(this.location.postalCode) ? this.location.postalCode : "";
	},
	
	/**
	 * Retourne l'URL qui permet d'accèder au poi
	 * @return {String} URL permettant d'accèder à la page du poi
	 */
	getURL : function () {
		if (!$isDefined(ViaMichelin.Api.Constants.System.ApiVMWebRoot)) return "";
		var url = ViaMichelin.Api.Constants.System.ApiVMWebRoot
			+ "/web/Datasheet"
			+ "/" + this.id
			+ "/" + this.productId;
		return url;
	},
	
	/**
	 * Retourne le type d'objet qu'est le poi MUP
	 * @return {String} Retourne le type d'objet du poi MUP
	 */
	getObjectType : function () {
		return "ressource";
	},
	
	/**
	 * Retourne le sous-type d'objet qu'est le poi MUP
	 * @return {String} Retourne le sous-type d'objet du poi MUP
	 */
	getObjectSubType : function () {
		return this.type;
	},
	
	/**
	 * Permet de vérifier que le poi est valide
	 * <br>Pour être considéré comme valide, il doit contenir les paramètres suivants :
	 * <br>- id
	 * <br>- type (database id)
	 * <br>- principalType
	 * <br>- secondType
	 * @return {Boolean} true si le poi est valide
	 */
	isValid : function () {
		return ($isNotEmpty(this.id) && $isNotEmpty(this.type) && $isNotEmpty(this.principalType) && $isNotEmpty(this.secondType));
	},
	
	// Méthodes permettant de tester le type de poi
	// --------------------------------------------
	
	/**
	 * Pemet de savoir si le poi correspond à un restaurant
	 * @return {Boolean} true si le poi est un restaurant, false sinon
	 */
	isPoiRestaurant: function () {
		return this.principalType == "restaurant"
				|| this.secondType == "gastronomy";
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un hotel
	 * @return {Boolean} true si le poi est un hotel, false sinon
	 */
	isPoiHotel: function () {
		return this.principalType == "accomodation"
				|| this.secondType == "hotel"
				|| this.isHotel == true;
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un site touristique
	 * @return {Boolean} true si le poi est un site touristique, false sinon
	 */
	isPoiTourism: function () { 
		return (
				this.principalType == "tourism"
				|| this.secondType == "tourism"
				)
				&& !this.isPoiLeisure()
				&& !this.isPoiMagazine();			
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un loisir
	 * @return {Boolean} true si le poi est un loisir, false sinon
	 */
	isPoiLeisure: function() {
		return this.principalType == "tourism"
				&& this.secondType == "loisir";
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un loisir
	 * @deprecated Utiliser isPoiLeisure à la place
	 * @return {Boolean} true si le poi est un loisir, false sinon
	 */
	isPoiLoisir : function() {
		return (this.type == ViaMichelin.Api.Constants.MPMPoi.LEISURE);
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un loisir
	 * @return {Boolean} true si le poi est un loisir, false sinon
	 */
	isPoiTravelBook : function() {
		return this.principalType == "travelBook"
				&& this.secondType == "travelBook";
	},
	
	/**
	 * Pemet de savoir si le poi correspond à un magazine
	 * @return {Boolean} true si le poi est un magazine, false sinon
	 */
	isPoiMagazine : function() {
		return this.principalType == "mag"
				&& this.secondType == "tourism";
	},
	
	/**
	 * @ignore
	 */
	toString : function(){
		return "[ViaMichelin.Api.Model.MUPPoi]";
	}
};